#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package services;
use strict;

use DBI;

use modules::serviceslibs::adminserv;
use modules::serviceslibs::nickserv;
use modules::serviceslibs::chanserv;
use modules::serviceslibs::operserv;
use modules::serviceslibs::botserv;
use modules::serviceslibs::memoserv;
use modules::serviceslibs::hostserv;

my $conffile = "$main::prefix/config/services.conf";
our %conf : shared;
our $dbh;
our @agents = (
	['NickServ', '+opqzBHS', 'Nick Registration Agent'],
	['ChanServ', '+pqzBS', 'Channel Registration Agent'],
	['OperServ', '+opqzBHS', 'Operator Services Agent'],
	['MemoServ', '+pqzBS', 'Memo Exchange Agent'],
	['BotServ', '+pqzBS', 'Channel Bot Control Agent'],
	['AdminServ', '+pqzBS', 'Services\' Administration Agent'],
	['HostServ', '+pqzBs', 'vHost Agent']
);
our $qlreason = 'Reserved for Services';

event::addhandler('SEOS', undef, undef, __PACKAGE__, 'services::ev_connect');
sub ev_connect {
	foreach my $a (@agents) {
		net::agent_connect($a->[0], 'services', undef, $a->[1], $a->[2]);
		net::sqline($a->[0], $qlreason);
		net::join($a->[0], $config::conf{diag});
		net::setmode($main::rsnick, $config::conf{diag}, '+o', $a->[0]);
	}
	#foreach my $a (@agents) {
		#net::agent_connect($a->[0], 'services', undef, $a->[1], $a->[2]);
		#net::sqline($a->[0], $qlreason);
	#}
	
	botserv::eos();
	nickserv::cleanup();
	nickserv::fix_vhosts();
	chanserv::eos();
	operserv::expire();
}

event::addhandler('EOS', undef, undef, __PACKAGE__, 'services::eos');
sub eos {
	chanserv::eos($_[0]);
}

event::addhandler('KILL', undef, undef, __PACKAGE__, 'nickserv::killhandle');

event::addhandler('NICKCONN', undef, undef, __PACKAGE__, 'services::ev_nickconn');
sub ev_nickconn {
    if($nickserv::enforcers{@_[0]}) {
	# Scream real loud about a collision
	# send a KILL
	# quit our agent and delete its timer
	#
	#btw this is more reason for there to be a kill handler for all agents!
    }
    
    nickserv::nick_create(@_[0,2..4,8,5,7,9,10]);
}

# NickServ
event::addhandler('NICKCHANGE', undef, undef, __PACKAGE__, 'nickserv::nick_change');
event::addhandler('QUIT', undef, undef, __PACKAGE__, 'nickserv::nick_delete');
event::addhandler('UMODE', undef, undef, __PACKAGE__, 'nickserv::umode');
event::addhandler('CHGHOST', undef, undef, __PACKAGE__, 'nickserv::chghost');
event::addhandler('CHGIDENT', undef, undef, __PACKAGE__, 'nickserv::chgident');
event::addhandler('USERIP', undef, undef, __PACKAGE__, 'nickserv::userip');
event::addhandler('SQUIT', undef, undef, __PACKAGE__, 'nickserv::squit') if $net::noquit;

event::addhandler('PRIVMSG', undef, lc $nickserv::nsnick, __PACKAGE__, 'nickserv::dispatch');

event::addhandler('BACK', undef, undef, __PACKAGE__, 'nickserv::notify_auths');

# ChanServ
event::addhandler('JOIN', undef, undef, __PACKAGE__, 'chanserv::user_join');
event::addhandler('SJOIN', undef, undef, __PACKAGE__, 'chanserv::handle_sjoin');
event::addhandler('PART', undef, undef, __PACKAGE__, 'chanserv::user_part');
event::addhandler('KICK', undef, undef, __PACKAGE__, 'chanserv::process_kick');
event::addhandler('MODE', undef, '#', __PACKAGE__, 'chanserv::chan_mode');
event::addhandler('TOPIC', undef, undef, __PACKAGE__, 'chanserv::chan_topic');

event::addhandler('PRIVMSG', undef, lc $chanserv::csnick, __PACKAGE__, 'chanserv::dispatch');

# OperServ
event::addhandler('PRIVMSG', undef, lc $operserv::osnick, __PACKAGE__, 'operserv::dispatch');

event::addtimer('flood_expire', 10, __PACKAGE__, 'operserv::flood_expire');

# MemoServ
event::addhandler('PRIVMSG', undef, lc $memoserv::msnick, __PACKAGE__, 'memoserv::dispatch');
event::addhandler('BACK', undef, undef, __PACKAGE__, 'memoserv::notify');

# BotServ
event::addhandler('PRIVMSG', undef, undef, __PACKAGE__, 'botserv::dispatch');
event::addhandler('NOTICE', undef, '#', __PACKAGE__, 'botserv::chan_msg');

# AdminServ
event::addhandler('PRIVMSG', undef, lc $adminserv::asnick, __PACKAGE__, 'adminserv::dispatch');

event::addtimer('', 30, __PACKAGE__, 'services::maint');
event::addtimer('', 20, __PACKAGE__, 'nickserv::cleanup_users');

# HostServ
event::addhandler('PRIVMSG', undef, lc $hostserv::hsnick, __PACKAGE__, 'hostserv::dispatch');

# $nick should be a registered root nick, if applicable
# $src is the nick or nickid that sent the command
sub ulog($$$$;$$) {
	my ($service, $level, $text) = splice(@_, 0, 3);
	
	my $hostmask = nickserv::get_hostmask($_[0]);

	# TODO - Record this in the database
	
	log::wlog($service, $level, "$hostmask - $text");
}

sub maint {
	log::wlog($main::rsnick, log::INFO(), " -- Running maintenance routines.");
#	net::privmsg($main::rsnick, $config::conf{diag}, " -- Running maintenance routines.");
	event::addtimer('', 3600, __PACKAGE__, 'services::maint');

	nickserv::expire();
	chanserv::expire();

	log::wlog($main::rsnick, log::INFO(), " -- Maintenance routines complete.");
#	net::privmsg($main::rsnick, $config::conf{diag}, " -- Maintenance routines complete.");
}

sub init {
	%conf = config::readHash($conffile);
	
	$dbh = DBI->connect("DBI:mysql:".$conf{'mysql-db'}, $conf{'mysql-user'}, $conf{'mysql-pass'}, {  AutoCommit => 1, RaiseError => 1 });

	$dbh->do("TRUNCATE TABLE chanuser");
	$dbh->do("TRUNCATE TABLE nickchg");
	$dbh->do("TRUNCATE TABLE chan");
	$dbh->do("TRUNCATE TABLE chanban");
	$dbh->do("UPDATE user SET online=0, quittime=".time());

	#$dbh->do("DROP TABLE IF EXISTS olduser");
	#$dbh->do("CREATE TABLE `newuser` (
	#	`id` int(10) unsigned NOT NULL default '0',
	#	`nick` char(30) NOT NULL default '',
	#	`time` int(10) unsigned NOT NULL default '0',
	#	`inval` tinyint(4) NOT NULL default '0',
	#	`ident` char(10) NOT NULL default '',
	#	`host` char(64) NOT NULL default '',
	#	`vhost` char(64) NOT NULL default '',
	#	`server` char(64) NOT NULL default '',
	#	`modes` char(30) NOT NULL default '',
	#	`gecos` char(50) NOT NULL default '',
	#	`guest` tinyint(4) NOT NULL default '0',
	#	PRIMARY KEY  (`id`), UNIQUE KEY `nick` (`nick`)) TYPE=HEAP");
	#$dbh->do("RENAME TABLE user TO olduser, newuser TO user");

	$dbh->disconnect;
}

sub begin {
	$dbh = DBI->connect(
		"DBI:mysql:".$conf{'mysql-db'}.":mysql_server_prepare=1",
		$conf{'mysql-user'},
		$conf{'mysql-pass'},
		{
			AutoCommit => 1,
			RaiseError => 0,
			mysql_auto_reconnect => 1,
		}
	);
	# Prevent timeout
	$dbh->do("SET wait_timeout=(86400*365)");

	nickserv::init();
	chanserv::init();
	operserv::init();
	botserv::init();
	adminserv::init();
	memoserv::init();
	hostserv::init();
}

sub end {
	$dbh->disconnect;
}

sub unload { }

1;
